package org.one.stone.soup.browser;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;

import org.one.stone.soup.file.FileHelper;
import org.one.stone.soup.file.ProgressListener;

/*
 * Wet -Wired.com Browser Version Date 27-Aug-2004
 *
 * Copyright 2000-2001 by Wet-Wired.com Ltd.,
 * Portsmouth England
 * This software is OSI Certified Open Source Software
 * This software is covered by an OSI approved open source licence
 * which can be found at http://www.wet-wired.co.uk/pages/licence-os.html
 */

/**
 * @author nikcross
 *
 */

public class BrowserURLFactory{

	public static final String FORM_BOUNDARY = "----moCtoDderiWteW";

	private static URL baseURL;

	private static String cookie = null;
private BrowserURLFactory(){}
protected static String getBase()
{
	if(baseURL==null)
		return null;

	return baseURL.toString();
}
protected static String getURL(String spec) throws MalformedURLException{

	URL url;

	try{
		url = new URL(spec);
	}
	catch(MalformedURLException mue)
	{
		try{
			url = new URL(baseURL,spec);
		}
		catch(MalformedURLException mue2)
		{
			throw mue;
		}
	}

//	System.out.println("URL: "+url);
	return url.toString();
}
protected static InputStream getURLInputStream(String browserName,String spec,String flavour,boolean isApplet) throws MalformedURLException,IOException{

	URL url;

	if(spec.length()>6 && spec.substring(0,6).equals("jar://"))
	{
		return Browser.class.getClassLoader().getResourceAsStream( spec.substring(6) );
	}

	try{
		url = new URL(spec);
	}
	catch(MalformedURLException mue)
	{
		try{
			url = new URL(baseURL,spec);
		}
		catch(MalformedURLException mue2)
		{
			throw mue;
		}
	}

	URLConnection connection = url.openConnection();
	connection.setRequestProperty( "User-Agent",browserName);
	if(cookie!=null)
	{
		connection.setRequestProperty( "Cookie",cookie);
	}

	return connection.getInputStream();
}
protected static InputStream getURLPost(String browserName,String spec,org.one.stone.soup.util.HashedVector data) throws MalformedURLException,IOException{

	URL url = null;

	try{
		url = new URL(spec);
	}
	catch(MalformedURLException mue)
	{
		try{
			url = new URL(baseURL,spec);
		}
		catch(MalformedURLException mue2)
		{
			throw mue;
		}
	}

	URLConnection connection = url.openConnection();

	connection.setDoInput(true);
	connection.setDoOutput(true);
	connection.setUseCaches(false);

//	connection.setRequestProperty("access","POST");
	connection.setRequestProperty( "Content-Type","application/x-www-form-urlencoded" );
	if(browserName!=null)
	{
		connection.setRequestProperty( "User-Agent",browserName);
	}
	if(cookie!=null)
	{
		connection.setRequestProperty( "Cookie",cookie);
	}
//	connection.connect();

	PrintWriter out = new PrintWriter( connection.getOutputStream() );

	StringBuffer dataString = new StringBuffer();
	for(int loop=0;loop<data.size();loop++)
	{
		if(loop>0)
			dataString.append('&');

		dataString.append( data.keyAt(loop)+"="+URLEncoder.encode( (String)data.elementAt(loop),"UTF-8" ) );
	}
	out.print( dataString.toString() );
	out.flush();

	String newCookie = connection.getHeaderField("Set-Cookie");
	if(newCookie!=null)
	{
		cookie = newCookie.substring(0,newCookie.indexOf(";"));
	}

	return connection.getInputStream();
}
protected static InputStream getURLPostFile(String browserName,String spec,String fileName,InputStream fileStream) throws MalformedURLException,IOException{

	URL url = null;

	try{
		url = new URL(spec);
	}
	catch(MalformedURLException mue)
	{
		try{
			url = new URL(baseURL,spec);
		}
		catch(MalformedURLException mue2)
		{
			throw mue;
		}
	}

	HttpURLConnection connection = (HttpURLConnection)url.openConnection();

	connection.setDoInput(true);
	connection.setDoOutput(true);
	connection.setUseCaches(false);

	connection.setRequestProperty( "Content-Type","multipart/form-data; boundary="+FORM_BOUNDARY );
	if(browserName!=null)
	{
		connection.setRequestProperty( "User-Agent",browserName);
	}
	if(cookie!=null)
	{
		connection.setRequestProperty( "Cookie",cookie);
	}
	
	OutputStream outStream = connection.getOutputStream();
	PrintWriter out = new PrintWriter( outStream );

	out.println("--"+FORM_BOUNDARY);
	out.println("Content-Disposition: form-data; name=\"filename\"; filename=\""+URLEncoder.encode(fileName,"UTF-8")+"\"");
	out.println("Content-Type: application/octet-stream");
	out.println();
	out.flush();

	BufferedInputStream fileBuffer = new BufferedInputStream(fileStream);
	byte[] dataBuffer = new byte[1000];
	int size = fileBuffer.read(dataBuffer);

	while(size>0)
	{
		outStream.write( dataBuffer,0,size );
		size = fileBuffer.read(dataBuffer);
		outStream.flush();
	}
	fileStream.close();

	out.print("\r\n--"+FORM_BOUNDARY+"--\r\n");
	out.flush();
	
	InputStream stream = connection.getInputStream();
	
	String newCookie = connection.getHeaderField("Set-Cookie");
	if(newCookie!=null)
	{
		cookie = newCookie.substring(0,newCookie.indexOf(";"));
	}
	
	outStream.flush();
	outStream.close();

	//connection.connect();

	return stream;
}

protected static URLConnection openURLPostFileConnection(String browserName,String spec) throws MalformedURLException,IOException{

	URL url = null;

	try{
		url = new URL(spec);
	}
	catch(MalformedURLException mue)
	{
		try{
			url = new URL(baseURL,spec);
		}
		catch(MalformedURLException mue2)
		{
			throw mue;
		}
	}

	HttpURLConnection connection = (HttpURLConnection)url.openConnection();

	connection.setDoInput(true);
	connection.setDoOutput(true);
	connection.setUseCaches(false);

	connection.setRequestProperty( "Content-Type","multipart/form-data; boundary="+FORM_BOUNDARY );
	if(browserName!=null)
	{
		connection.setRequestProperty( "User-Agent",browserName);
	}
	if(cookie!=null)
	{
		connection.setRequestProperty( "Cookie",cookie);
	}

	return connection;
}

protected static void closeURLPostFileConnection(URLConnection connection,OutputStream outStream) throws IOException
{
	PrintWriter out = new PrintWriter( outStream );

	out.print("\r\n--"+FORM_BOUNDARY+"--\r\n");
	out.flush();
	outStream.flush();
	
	String newCookie = connection.getHeaderField("Set-Cookie");
	if(newCookie!=null)
	{
		cookie = newCookie.substring(0,newCookie.indexOf(";"));
	}
	
	outStream.flush();
	outStream.close();

	InputStream inStream = connection.getInputStream();
	String result = new String(FileHelper.readFile(inStream));
	System.out.println("Result:"+result);
}

protected static boolean isBaseSet() {
	if(baseURL==null)
		return false;
	else
		return true;
}
protected static InputStream setBase(String newSpec,String flavour,ProgressListener listener,boolean isApplet) throws IOException
{
	String spec = newSpec;

	if(spec.length()>6 && spec.substring(0,6).equals("jar://"))
	{
		return Browser.class.getClassLoader().getResourceAsStream( spec.substring(6) );
	}
	
	try{
		baseURL = new URL(getURL(spec));
	}
	catch(Exception e)
	{
		if(!isApplet && baseURL==null)
		{
			spec = System.getProperty("user.dir")+spec;
			spec = new File(spec).toURL().toExternalForm();
			baseURL = new URL(getURL(spec));
		}
	}

	URLConnection connection = baseURL.openConnection();
	connection.setRequestProperty( "User-Agent",flavour);
	if(cookie!=null)
	{
		connection.setRequestProperty( "Cookie",cookie);
	}

	return connection.getInputStream();
}
}
