package org.one.stone.soup.mjdb.data.group;

import org.one.stone.soup.mjdb.data.field.*;
import java.util.*;

/*
 * MJDB (Micro Java Database) Version 1.0
 *
 * Copyright 2000-2001 by Wet-Wired.com Ltd.,
 * Portsmouth England
 * This software is OSI Certified Open Source Software
 * This software is covered by an OSI approved open source licence
 * which can be found at http://www.wet-wired.com/licence/license-os.html
 */

public class Table extends Field implements DataGroupInterface,FieldListener{
	private String name = "Table";

	private int noOfColumns=0;
	private int noOfRows=0;

	private Row columnTitles;
	private Row columnTypes;
	private Row columnConstraints;

	private Vector rows = new Vector();

public Table() {
}
public Table(int noColumns) {

	try{
		setNoOfColumns(noColumns,this);
	}
	catch(LockException e){}//Dummy exception
	catch(SetException e){}//Dummy exception
}
public int compare(Field target) throws CompareException {
	return 0;
}
public void dump() {

	System.out.println("Titles:");
	for(int loop=0;loop<getNoOfColumns();loop++)
	{
		System.out.println(loop+": "+getColumnTitle(loop));
	}

	System.out.println("\nData:");
	for(int loopR=0;loopR<getNoOfRows();loopR++)
	{
	for(int loopC=0;loopC<getNoOfColumns();loopC++)
	{
		try{
			System.out.print(getField(loopC,loopR)+"\t");
		}
		catch(DatabaseException de)
		{
			de.printStackTrace();
		}
	}
		System.out.println("");
	}
}
public int findInCol(int col,String data) throws DatabaseException{
	Object target;

	if(col>=noOfColumns)
		return -1;

	for(int loop=0;loop<noOfRows;loop++)
	{
		target=getField(col,loop);

		if(target.toString().equals(data))
				return loop;
	}
	return -1;
}
public Object getColumnTitle(int colNo) {
	return columnTitles.getField(colNo);
}
public int getColumnTitleIndex(String titleName) {

	int index = -1;

	for(int loop=0;loop<getNoOfColumns();loop++)
	{
		if(titleName.equals(columnTitles.getField(loop).toString()))
		{
			index=loop;
			break;
		}
	}

	return index;
}
public Row getColumnTitles() {
	return columnTitles;
}
public Object getColumnType(int colNo) {
	return columnTypes.getField(colNo);
}
public Object getColumnTypes(int colNo) {
	return columnTypes.getField(colNo);
}
public Object getField(int col,int row) throws DatabaseException{

	if(col>=noOfColumns || row>=noOfRows)
	{
		throw new DatabaseException("getField column:"+col+" row:"+row+" out of range");
	}

	return ((Row)rows.elementAt(row)).getField(col);
}
public Object getField(String columnTitle,int row) throws DatabaseException {

	int col = getColumnTitleIndex(columnTitle);

	if(col>=noOfColumns || row>=noOfRows)
	{
		throw new DatabaseException("getField column:"+columnTitle+" row:"+row+" not found");
	}

//	Row temp = (Row)(rows.elementAt(row));

	//System.out.println("getField("+col+","+row+")="+temp.getField(col));

	return ((Row)rows.elementAt(row)).getField(col);
}
public String getName()
{
	return name;
}
public int getNoOfColumns() {
	return noOfColumns;
}
public int getNoOfRows() {
	return noOfRows;
}
public Row getRow(int index) {
	return (Row)rows.elementAt(index);
}
/**
 * getValue method comment.
 */
public String getValue(String name) {
	return null;
}
public void setColumnTitle(int colNo,Object name,Object source) throws LockException,SetException {

	if(locked && !isLockOwner(source))
		throw new LockException(this+"is locked");

	columnTitles.setField(colNo,name,source);

	if(report)
		reportChange(source);
}
public void setColumnType(int colNo,Object name,Object source) throws LockException,SetException {

	if(locked && !isLockOwner(source))
		throw new LockException(this+"is locked");

	columnTypes.setField(colNo,name,source);

	if(report)
		reportChange(source);
}
public void setField(int col,int row,Object data,Object source) throws LockException,SetException,DatabaseException {

	if(locked && !isLockOwner(source))
		throw new LockException(this+"is locked");

	if(col>=noOfColumns || row>=noOfRows)
		throw new DatabaseException("setField column:"+col+" row:"+row+" out of range");

	Row temp = (Row)(rows.elementAt(row));
	temp.setField(col,data,source);

	if(report)
		reportChange(source);
}
public void setField(int col,Object data,Object source) throws LockException,SetException,DatabaseException {

	if(locked && !isLockOwner(source))
		throw new LockException(this+"is locked");

	if(col>=noOfColumns)
		throw new DatabaseException("setField column:"+col+" out of range");

	Row temp = (Row)(rows.elementAt(getNoOfRows()-1));
	temp.setField(col,data,this);

	if(report)
		reportChange(source);
}
public void setField(String columnTitle,int row,Object data,Object source) throws LockException,SetException,DatabaseException {

	int col = getColumnTitleIndex(columnTitle);

	if(locked && !isLockOwner(source))
		throw new LockException(this+"is locked");

	if(col>=noOfColumns || row>=noOfRows)
		throw new DatabaseException("setField column:"+columnTitle+" row:"+row+" not found");

	Row temp = (Row)(rows.elementAt(row));
	temp.setField(col,data,source);

	if(report)
		reportChange(source);
}
public void setName(String name)
{
	this.name = name;
}
public void setNoOfColumns(int noColumns,Object source) throws LockException,SetException {

	if(locked && !isLockOwner(source))
		throw new LockException(this+"is locked");

	noOfColumns = noColumns;
	columnTitles=new Row(noOfColumns,false);
	columnTypes=new Row(noOfColumns,false);

	if(report)
		reportChange(source);
}
public void setRow(Row temp,int pos,Object source) throws LockException,SetException {

	if(locked && !isLockOwner(source))
		throw new LockException(this+"is locked");

	rows.setElementAt(temp,pos);

	if(report)
		reportChange(source);
}
/**
 * setValue method comment.
 */
public void setValue(String name, String value, Object source) throws LockException, ConstructException {}
public String toString()
{
	return name+" Rows:"+getNoOfRows()+" Cols:"+getNoOfColumns();
}

public void addColumn(Object columnTitle,Object source) throws SetException,LockException
{
	if(locked)
		throw new LockException(this+"is locked");

	noOfColumns++;

	if(columnTitles==null)
		columnTitles = new Row(0);
	if(columnTypes==null)
		columnTypes = new Row(0);

	columnTitles.addField(columnTitle,this);
	columnTypes.addField(null,this);

	for(int loopR=0;loopR<getNoOfRows();loopR++)
	{
		getRow(loopR).addField(true,this);
	}

	if(report)
		reportChange(source);
}

public Row addRow(Object source) throws LockException
{
	if(locked && !isLockOwner(source))
		throw new LockException(this+"is locked");

	Row temp = new Row(noOfColumns);
	rows.addElement(temp);
	noOfRows++;

	if(report)
		reportChange(source);

	return temp;
}

public void addRow(Row temp,Object source) throws LockException
{
	if(locked)
		throw new LockException(this+"is locked");

	rows.addElement(temp);
	noOfRows++;

	if(report)
		reportChange(source);
}

public void addTable(Table temp,Object source) throws DatabaseException,LockException
{
	if(locked)
		throw new LockException(this+"is locked");

	if(temp.getNoOfColumns() != getNoOfColumns())
		throw new DatabaseException("Table to be added has wrong number of columns.");

	for(int loop=0;loop<temp.getNoOfRows();loop++)
	{
		addRow(temp.getRow(loop),source);
	}

	if(report)
		reportChange(source);
}

public void autoWrap(Object source) throws SetException,LockException {
	if(locked && !isLockOwner(source))
		throw new LockException(this+"is locked");

	for(int loopR=0;loopR<noOfRows;loopR++)
	{
		getRow(loopR).autoWrap(source);
	}

	if(report)
		reportChange(source);
}

public void autoWrapAsStrings(Object source) throws SetException,LockException {
	if(locked)
		throw new LockException(this+"is locked");

	for(int loopR=0;loopR<noOfRows;loopR++)
	{
		getRow(loopR).autoWrapAsStrings(source);
	}

	if(report)
		reportChange(source);
}

public void insertRow(Row temp,int pos,Object source) throws LockException
{
	if(locked && !isLockOwner(source))
		throw new LockException(this+"is locked");

	rows.insertElementAt(temp,pos);
	noOfRows++;

	if(report)
		reportChange(source);
}

public void removeColumn(int index,Object source) throws LockException {

	if(locked && !isLockOwner(source))
		throw new LockException(this+"is locked");

	noOfColumns--;

	if(columnTitles!=null)
	{
		columnTitles.removeField(index,source);
	}
	if(columnTypes!=null)
	{
		columnTypes.removeField(index,source);
	}

	for(int loopR=0;loopR<getNoOfRows();loopR++)
	{
		getRow(loopR).removeField(index,source);
	}

	if(report)
		reportChange(source);
}

public void removeRow(int pos,Object source) throws LockException
{

	if(locked && !isLockOwner(source))
		throw new LockException(this+"is locked");

	rows.removeElementAt(pos);
	noOfRows--;

	if(report)
		reportChange(source);
}

public void removeRow( Row row,Object source) throws LockException
{
	if(locked && !isLockOwner(source))
		throw new LockException(this+"is locked");

	rows.removeElement( row );
	noOfRows--;

	if(report)
		reportChange(source);
}
public void addListener(FieldListener listener)
	{
		super.addListener( listener );
		for(int loop=0;loop<getNoOfRows();loop++)
		{
			getRow(loop).addListener(this);
		}
	}
/* (non-Javadoc)
 * @see org.one.stone.soup.mjdb.data.field.FieldListener#receiveChange(java.lang.Object, java.lang.Object)
 */
public void receiveChange(Object source, Object data) {
	reportChange(source);}
}
