package org.one.stone.soup.remote.control.client;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;

import javax.swing.JButton;
import javax.swing.JOptionPane;

import org.one.stone.soup.screen.recorder.ScreenPlayer;
import org.one.stone.soup.swing.JRootFrame;
import org.one.stone.soup.xml.XmlDocument;
import org.one.stone.soup.xml.XmlElement;
import org.one.stone.soup.xml.stream.XmlLoader;

/**
 * @author Nicholas Cross
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class RemoteControlClientUI extends JRootFrame implements ActionListener,MouseListener,MouseMotionListener,KeyListener,RemoteClientController{

	private JButton connectButton;

	private boolean fullscreen = false;
	private ScreenPlayer screenPlayer; 
	private RemoteControlClientFrame client;
	private String serverKey;
	
	private Socket mainSocket;
	private Socket controlSocket;

	private String hubAlias;				

	private String serverAlias;	
	private String address;
	private int port = 169;
	private boolean useHub=false;

	
	public static final void main(String[] args)
	{
		new RemoteControlClientUI(args);
	}
	
	public RemoteControlClientUI(String[] args)
	{
		super("Remote Control Client",args);
		
		hubAlias = JOptionPane.showInputDialog(this,"Enter Your Alias");
		if(hubAlias==null)
		{
			System.exit(1);
		}
		
		initialiseUI();
	}
	
	private void initialiseUI()
	{
		connectButton = new JButton("Connect");
		connectButton.setActionCommand("connect");
		connectButton.addActionListener(this);
		getContentPane().add(connectButton);

		this.pack();
		this.setVisible(true);
	}
	
	private boolean getAddress()
	{
		address = JOptionPane.showInputDialog(this,"Enter Server Alias");
		if(address==null)
		{
			connectButton.setEnabled(true);
			return false;			
		}
		
		// alias=nnnnnnn@address:port
		if( address.indexOf("@")>0 )
		{
			useHub = true;
			serverAlias = address.substring( 0,address.indexOf("@") );				
			address = address.substring( address.indexOf("@")+1 );
		}
		else
		{
			serverAlias = address;
		}
		
		if(address.indexOf(":")>0)
		{
			port = Integer.parseInt( address.substring( address.indexOf(":")+1 ));
			address = address.substring( 0,address.indexOf(":"));
		}
		
		return true;
	}
	
	private void loginToHub(String subdomain,Socket socket,String password) throws IOException
	{
		writeLine(subdomain,socket.getOutputStream());
		String line = readLine(socket.getInputStream());
		
		if(hubAlias==null)
		{
			hubAlias = JOptionPane.showInputDialog(this,"Enter Your Alias");
		}
		if(password==null)
		{
			password = JOptionPane.showInputDialog(this,"Enter Hub Login Password");
		}			
		writeLine( hubAlias+":"+password,socket.getOutputStream() );
		
		line = readLine(socket.getInputStream());
		if(line.equals("BYE"))
		{
			socket.close();
			JOptionPane.showMessageDialog(this,"Hub Login Failed");
			hubAlias = null;
			password=null;
			return;
		}
		else if(( line.indexOf("HELLO ")==0 ))
		{
			serverKey = line.substring( line.indexOf("HELLO ")+6 );
		}

		writeLine(serverAlias,mainSocket.getOutputStream());		
	}
	
	private void connect()
	{
		connectButton.setEnabled(false);
		
		if( getAddress()==false )
		{
			return;
		}
		
		try{
			mainSocket = new Socket(address,port);
		
			client = new RemoteControlClientFrame(this);
			if(useHub==true)
			{
				loginToHub("client.screen",mainSocket,null);
			}
			
			writeLine(hubAlias,mainSocket.getOutputStream());
			
			InputStream iStream = mainSocket.getInputStream();

			String targetKey = readLine(iStream);
			if(targetKey.length()==0)
			{
				JOptionPane.showMessageDialog(this,"Connection refused");
				disconnect();
				return;
			}
			
			//System.out.println("Key:"+key);
			
			screenPlayer = new ScreenPlayer(iStream,client);
			
			connectButton.setText("Disconnect");
			connectButton.setActionCommand("disconnect");
			connectButton.setEnabled(true);
						
			controlSocket = new Socket(address,port);
			
			if(useHub==true)
			{
				loginToHub("client.control",controlSocket,serverKey);
			}

			writeLine(targetKey,controlSocket.getOutputStream());							

			if( JOptionPane.showConfirmDialog(this,"Take Control of remote machine ?")==JOptionPane.OK_OPTION )
			{
				writeLine("fullControl",controlSocket.getOutputStream());

				client.getView().addMouseListener(this);
				client.getView().addMouseMotionListener(this);
				client.getView().addKeyListener(this);
				client.addKeyListener(this);
			}
			else
			{
				writeLine("flowControl",controlSocket.getOutputStream());					
			}
		
			System.out.println("Screen Player Started");
			
			screenPlayer.play();	
		}
		catch(Exception e)
		{ disconnect(); }
		
	}
	
	private void disconnect()
	{
		try{
			client.getView().removeMouseListener(this);
			client.getView().removeMouseMotionListener(this);
			client.getView().removeKeyListener(this);
		}catch(Exception e){}
		
		try{ mainSocket.close(); }catch(Exception e){}
		try{ controlSocket.close(); }catch(Exception e){}
		
		screenPlayer=null;
		
		try {
			client.dispose();
			client = null;
		}catch(Exception e){}
		
		try{
			connectButton.setText("Connect");
			connectButton.setActionCommand("connect");
			connectButton.setEnabled(true);	
		}catch(Exception e){}
		JOptionPane.showMessageDialog(this,"Disconnected from Remote Machine");
	}
	
	/* (non-Javadoc)
	 * @see wet.wired.swing.JRootFrame#destroy()
	 */
	public boolean destroy() {
		return true;
	}

	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent action) {
		
		if(action.getActionCommand().equals("connect"))
		{
			connect();
		}
		else if(action.getActionCommand().equals("disconnect"))
		{
			disconnect();
		}
	}
	
	public void keyPressed(KeyEvent e) 
	{
		if(e.getKeyCode()==KeyEvent.VK_F11)
		{
			if(client!=null)
			{
				fullscreen = !fullscreen;
				client.setFullscreen(fullscreen);
				
				return;
			}
		}
		
		XmlElement packet = new XmlElement("keyPressed");
		packet.addAttribute("code",""+e.getKeyCode());
		sendCommand(packet);
	}
	public void keyReleased(KeyEvent e)
	{
		XmlElement packet = new XmlElement("keyReleased");
		packet.addAttribute("code",""+e.getKeyCode());
		sendCommand(packet);
	}
	public void mouseDragged(MouseEvent e)
	{
		mouseMoved(e);
	}
	public void mouseMoved(MouseEvent e)
	{	
		XmlElement packet = new XmlElement("mouseMoved");
		packet.addAttribute("x",""+e.getX());
		packet.addAttribute("y",""+e.getY());
		sendCommand(packet);
	}
	public void mousePressed(MouseEvent e)
	{
		XmlElement packet = new XmlElement("mousePressed");
		packet.addAttribute("code",""+e.getModifiers());
		sendCommand(packet);
	}
	public void mouseReleased(MouseEvent e)
	{
		XmlElement packet = new XmlElement("mouseReleased");
		packet.addAttribute("code",""+e.getModifiers());
		sendCommand(packet);
	}
	
	public void requestNextFrame()
	{
		XmlElement packet = new XmlElement("nextFrame");
		sendCommand(packet);
		
		try{
			XmlDocument result = XmlLoader.load( controlSocket.getInputStream() );
			System.out.println(result);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
	
	private synchronized void sendCommand(XmlElement packet)
	{
		if(controlSocket==null)
		{
			return;
		}
		try{
			controlSocket.getOutputStream().write(packet.toXmlNoBeautify().getBytes());
			controlSocket.getOutputStream().flush();
			
			//System.out.println("Sent: "+packet.toXmlNoBeautify());
		}
		catch(Exception e){disconnect();}
	}
	
	public static String readLine(InputStream iStream) throws IOException
	{
		int in = iStream.read();
		StringBuffer buffer = new StringBuffer();
		while(in!=-1 && in!='\n' && buffer.length()<100)
		{
			if(in!=0) // keep alive byte
			{
				buffer.append((char)in);
			}
						
			in = iStream.read();
		}
		
		return buffer.toString();
	}
	
	public static void writeLine(String line,OutputStream outputStream) throws IOException
	{
		outputStream.write( (line+"\n").getBytes() );
		outputStream.flush();				
	}
}
