package org.one.stone.soup.xapp.components.spreadsheet;

import java.util.Vector;

import org.one.stone.soup.xapp.XappActionAdapter;
import org.one.stone.soup.xapp.XappRootApplication;
import org.one.stone.soup.xapp.factories.XappActionAdapterFactory;
import org.one.stone.soup.xapp.factories.XappMenuFactory;
import org.one.stone.soup.xml.XmlElement;

public class XappSpreadsheetAdapter extends XappActionAdapter {

	private static String currentSheetName;
	
	public static void setCurrentSheetName(String name)
	{
		currentSheetName = name;
	}
	
	public XappSpreadsheetAdapter(XmlElement definition)
	{
		super(definition);
	}
	
	public XmlElement getMenu() {
		XmlElement xMenu = XappMenuFactory.getMenuDefinition("Spreadsheet");
		XappMenuFactory.addSubMenuItem(xMenu,"Row","Add","addRow");
		XappMenuFactory.addSubMenuItem(xMenu,"Row","Remove","removeRow");
		XappMenuFactory.addSubMenuItem(xMenu,"Row","Move Up","moveRowUp");
		XappMenuFactory.addSubMenuItem(xMenu,"Row","Move Down","moveRowDown");
		XappMenuFactory.addSubMenuItem(xMenu,"Row","Insert","insertRow");
		XappMenuFactory.addSubMenuItem(xMenu,"Column","Add","addColumn");
		XappMenuFactory.addSubMenuItem(xMenu,"Column","Rename","renameColumn");
		XappMenuFactory.addSubMenuItem(xMenu,"Column","Insert","insertColumn");
		XappMenuFactory.addSubMenuItem(xMenu,"Column","Remove","removeColumn");
		
		return xMenu;
	}

	public XmlElement getActions() {
		XmlElement xActions = XappActionAdapterFactory.getActionsDefinition();
		XappActionAdapterFactory.addAction(this,xActions,"addColumn");
		XappActionAdapterFactory.addAction(this,xActions,"addRow");
		XappActionAdapterFactory.addAction(this,xActions,"removeColumn");
		XappActionAdapterFactory.addAction(this,xActions,"removeRow");
		XappActionAdapterFactory.addAction(this,xActions,"moveRowUp");
		XappActionAdapterFactory.addAction(this,xActions,"moveRowDown");
		XappActionAdapterFactory.addAction(this,xActions,"renameColumn");
		XappActionAdapterFactory.addAction(this,xActions,"insertColumn");
		XappActionAdapterFactory.addAction(this,xActions,"insertRow");
		
		return xActions;
	}

	public void addColumn()
	{
		XappSpreadsheet sheet = (XappSpreadsheet)XappRootApplication.getComponent(currentSheetName);
		
		String newName = XappRootApplication.askForValue("New Column Name",currentSheetName);
		
		XmlElement xData = sheet.getData();
		xData.getElementByName("columns").addChild("title").addValue(newName);
		
		Vector rows = xData.getElementsByName("row");
		for(int loop=0;loop<rows.size();loop++)
		{
			XmlElement row = (XmlElement)rows.elementAt(loop);
			row.addChild("field").addValue("");
		}
		
		sheet.setData(xData);
	}

	public void insertColumn()
	{
		String position = XappRootApplication.askForSelection( "Select","Insert Column",new String[]{"Left","Right"},"Right" );
		if(position==null)
		{
			return;
		}		
		
		XappSpreadsheet sheet = (XappSpreadsheet)XappRootApplication.getComponent(currentSheetName);
		int column = sheet.getSelectedColumn();
		
		String newName = XappRootApplication.askForValue("New Column Name","");
		
		XmlElement xData = sheet.getData();
		xData.getElementByName("columns").addChild("title").addValue(newName);
		
		Vector rows = xData.getElementsByName("row");
		for(int loop=0;loop<rows.size();loop++)
		{
			XmlElement row = (XmlElement)rows.elementAt(loop);
			
			XmlElement field  = new XmlElement("field");
			field.addValue("");
			if(position.equals("Left"))
			{
				row.insertChild(field,column);
			}
			else if(position.equals("Right"))
			{
				row.insertChild(field,column+1);				
			}
		}
		
		sheet.setData(xData);
	}	
	
	public void removeColumn()
	{
		XappSpreadsheet sheet = (XappSpreadsheet)XappRootApplication.getComponent(currentSheetName);
		
		int columnCount = sheet.getColumnCount()-1;
		String[] options = new String[columnCount];
		for(int loop=0;loop<columnCount;loop++)
		{
			options[loop]=sheet.getColumnName(loop+1);
		}
		
		String newName = XappRootApplication.askForSelection("Name of Column to Remove","Select Column",options,options[0]);
		if(newName==null)
		{
			return;
		}
		
		XmlElement xData = sheet.getData();
		Vector columns = xData.getElementByName("columns").getElements();
		int index = -1;
		for(int loop=0;loop<columns.size();loop++)
		{
			XmlElement title = (XmlElement)columns.elementAt(loop);
			if( title.getValue().equals(newName) )
			{
				xData.getElementByName("columns").removeChild(title);
				index = loop;
			}
		}
		
		if(index==-1)
		{
			return;
		}
		
		Vector rows = xData.getElementsByName("row");
		for(int loop=0;loop<rows.size();loop++)
		{
			XmlElement row = (XmlElement)rows.elementAt(loop);
			row.removeData(index);
		}
		
		sheet.setData(xData);
	}
	
	public void removeRow()
	{
		XappSpreadsheet sheet = (XappSpreadsheet)XappRootApplication.getComponent(currentSheetName);
		int row = sheet.getSelectedRow();
		
		if(row<0)
		{
			return;
		}
		
		XmlElement xData = sheet.getData();
		XmlElement xRemove = xData.getElementByIndex(row+1);
		xData.removeChild(xRemove);
		
		sheet.setData(xData);
	}
	
	public void moveRowUp()
	{
		XappSpreadsheet sheet = (XappSpreadsheet)XappRootApplication.getComponent(currentSheetName);
		int row = sheet.getSelectedRow();
		
		if(row<0)
		{
			return;
		}
		
		XmlElement xData = sheet.getData();
		XmlElement xMove = xData.getElementByIndex(row+1);
		xData.removeChild(xMove);
		xData.insertChild(xMove,row);
		
		sheet.setData(xData);
	}
	
	public void moveRowDown()
	{
		XappSpreadsheet sheet = (XappSpreadsheet)XappRootApplication.getComponent(currentSheetName);
		int row = sheet.getSelectedRow();
		
		if(row<0 || row+2>sheet.getRowCount())
		{
			return;
		}
		
		XmlElement xData = sheet.getData();
		XmlElement xMove = xData.getElementByIndex(row+1);
		xData.removeChild(xMove);
		xData.insertChild(xMove,row+2);
		
		sheet.setData(xData);
	}		
	
	public void addRow()
	{
		XappSpreadsheet sheet = (XappSpreadsheet)XappRootApplication.getComponent(currentSheetName);
		
		XmlElement xData = sheet.getData();
		int columns = xData.getElementByName("columns").getElementCount();
		XmlElement xRow = xData.addChild("row");
		for(int loop=0;loop<columns;loop++)
		{
			xRow.addChild("field");
		}
		
		sheet.setData(xData);
	}
	
	public void renameColumn()
	{
		if(currentSheetName==null)
		{
			return;
		}
		XappSpreadsheet sheet = (XappSpreadsheet)XappRootApplication.getComponent(currentSheetName);
		
		int columnCount = sheet.getColumnCount()-1;
		String[] options = new String[columnCount];
		for(int loop=0;loop<columnCount;loop++)
		{
			options[loop]=sheet.getColumnName(loop+1);
		}
		
		String newName = XappRootApplication.askForSelection("Name of Column to Rename","Select Column",options,options[0]);
		
		XmlElement xData = sheet.getData();
		Vector columns = xData.getElementByName("columns").getElements();
		XmlElement title = null;
		for(int loop=0;loop<columns.size();loop++)
		{
			title = (XmlElement)columns.elementAt(loop);
			if( title.getValue().equals(newName) )
			{
				break;
			}
		}
		
		if(title==null)
		{
			return;
		}
		
		newName = XappRootApplication.askForValue("Enter new Column Name","");
		title.setValue(newName,0);
		
		sheet.setData(xData);
	}
	
	public void insertRow()
	{
		String position = XappRootApplication.askForSelection("Select","Insert Row",new String[]{"Above","Below"},"Above" );
		if(position==null)
		{
			return;
		}
		XappSpreadsheet sheet = (XappSpreadsheet)XappRootApplication.getComponent(currentSheetName);
		XmlElement xData = sheet.getData();
		int columns = xData.getElementByName("columns").getElementCount();
		int row = sheet.getSelectedRow();
		XmlElement xRow = new XmlElement("row"); 
		for(int loop=0;loop<columns;loop++)
		{
			xRow.addChild("field");
		}		

		if(position.equals("Above"))
		{	
			xData.insertChild(xRow,row);	
		}
		else if(position.equals("Below"))
		{
			xData.insertChild(xRow,row+1);		
		}		


		sheet.setData(xData);	
	}
}
