package org.one.stone.soup.xapp.controller.xml;

import java.util.Vector;

import org.one.stone.soup.xapp.xpath.XappXPathDB.XappSubXPathDB;
import org.one.stone.soup.xml.XmlElement;
import org.one.stone.soup.xml.helper.XmlHelper;
import org.one.stone.soup.xml.helper.XmlPathSearch;

public class XmlSchemaController{

	public XappSubXPathDB schemaDB;
	
	public XmlSchemaController getControllerFor(XmlElement root,XmlElement target)
	{
		//TODO
		return this;
	}
	
	public XmlSchemaController(XappSubXPathDB schemaDB)
	{
		this.schemaDB = schemaDB;
	}
	
	public XmlElement getPossibleAttributes(XmlElement root,XmlElement target)
	{
		XmlElement xDefinition = getDefinitionFor( target.getName() );
		
		XmlElement xAttributes = new XmlElement("attributes");
		if(xDefinition==null)
		{
			return xAttributes;
		}
		
		Vector attributes =xDefinition.getElementsByName("attribute");
		
		for(int loop=0;loop<attributes.size();loop++)
		{
			XmlElement xAttribute = (XmlElement)attributes.elementAt(loop);
			
			if(target.getAttributeValueByName(xAttribute.getAttributeValueByName("name"))==null)
			{
				xAttributes.addChild(xAttribute);
			}
		}
		
		return xAttributes;
	}
	
	public boolean canAddChild(XmlElement root,XmlElement target,XmlElement child)
	{		
		XmlElement possibleChildren = getPossibleChildren(root,target);
		
		for(int loop=0;loop<possibleChildren.getElementCount();loop++)
		{
			XmlElement match = possibleChildren.getElementByIndex(loop);
			
			if(match.getAttributeValueByName("name").equals(child.getName()))
			{
				return true;
			}
		}
		
		return false;
	}
	
	public XmlElement getPossibleChildren(XmlElement root,XmlElement target)
	{
		XmlElement xDefinition = getDefinitionFor(target.getName());
		XmlElement xNodes = new XmlElement("nodes");

		String sMaxChildren = xDefinition.getAttributeValueByName("maxChildren");
		if(sMaxChildren!=null)
		{
			int maxChildren = Integer.parseInt(sMaxChildren);
			if(target.getElementCount()>=maxChildren)
			{
				return xNodes;
			}
		}
		
		Vector nodes = xDefinition.getElementsByName("child");
		
		for(int loop=0;loop<nodes.size();loop++)
		{
			XmlElement xNode = (XmlElement)nodes.elementAt(loop);
			
			int instances = target.getElementsByName( xNode.getAttributeValueByName("name") ).size();
			int	min = getMinimumInstances( xNode );
			int max = getMaximumInstances( xNode );
			if(max<0 || instances<max)
			{
				xNodes.addChild(xNode);
			}
		}
		
		return xNodes;
	}
	
	public XmlElement addChild(XmlElement root,XmlElement target,String nodeName)
	{
		XmlElement definition = getDefinitionFor(target.getName(),nodeName);
		//XmlElement parent = getParent(root,target);
		//int instances = parent.getElementsByName( target.getName() ).size();
		int instances = target.getElementsByName(nodeName).size();
		int	max = getMaximumInstances( definition );
		if(instances>=max && max!=-1)
		{
			return null;
		}
		
		return createChild( target,definition );
	}
	
	public XmlElement addChild(XmlElement root,XmlElement target,XmlElement child)
	{
		if( canAddChild(root,target,child) )
		{
			target.addChild(child);
			return child;
		}
		return null;
	}	
	
	public void addAttribute(XmlElement root,XmlElement target,String attributeName)
	{
		XmlElement xAttribute = getAttributeDefinitionFor(target.getName(),attributeName);
		
		String value = xAttribute.getAttributeValueByName("default");
		if(value==null)
		{
			value="";
		}
		target.addAttribute(xAttribute.getAttributeValueByName("name"),value);
	}	
	
	public XmlElement createRoot()
	{
		XmlElement rootDefinition = schemaDB.findElement("schema/node[@root='true']");
		XmlElement root = null;
		if(rootDefinition==null)
		{
			root = new XmlElement("undefined");
			return root;
		}
		else
		{
			root = new XmlElement(rootDefinition.getAttributeValueByName("name"));
		}
		
		Vector attributes = getDefinitionFor( root.getName() ).getElementsByName("attribute");
		for(int loop=0;loop<attributes.size();loop++)
		{
			XmlElement xAttribute = (XmlElement)attributes.elementAt(loop);
			
			String required = xAttribute.getAttributeValueByName("required");
			if(required!=null && required.equals("true"))
			{
				String value = xAttribute.getAttributeValueByName("default");
				if(value==null)
				{
					value="";
				}
				root.addAttribute(xAttribute.getAttributeValueByName("name"),value);
			}
		}		
		
		Vector nodes = getDefinitionFor( root.getName() ).getElementsByName("child");
		for(int loop=0;loop<nodes.size();loop++)
		{
			XmlElement xNode = (XmlElement)nodes.elementAt(loop);

			int	min = getMinimumInstances( xNode );
			for(int loopA=0;loopA<min;loopA++)
			{
				createChild( root,getDefinitionFor(root.getName(),xNode.getAttributeValueByName("name")) );
			}
		}
		
		return root;
	}
	
	private XmlElement createChild(XmlElement target,XmlElement childDefinition)
	{
		XmlElement child = target.addChild(childDefinition.getAttributeValueByName("name"));
		
		Vector attributes = getDefinitionFor( child.getName() ).getElementsByName("attribute");
		for(int loop=0;loop<attributes.size();loop++)
		{
			XmlElement xAttribute = (XmlElement)attributes.elementAt(loop);
			
			String required = xAttribute.getAttributeValueByName("required");
			if(required!=null && required.equals("true"))
			{
				String value = xAttribute.getAttributeValueByName("default");
				if(value==null)
				{
					value="";
				}
				child.addAttribute(xAttribute.getAttributeValueByName("name"),value);
			}
		}		
		
		Vector nodes = getDefinitionFor( child.getName() ).getElementsByName("child");
		for(int loop=0;loop<nodes.size();loop++)
		{
			XmlElement xNode = (XmlElement)nodes.elementAt(loop);

			int	min = getMinimumInstances( xNode );
			for(int loopA=0;loopA<min;loopA++)
			{
				createChild( child,getDefinitionFor(child.getName(),xNode.getAttributeValueByName("name")) );
			}
		}
		
		return child;
	}
	
	public boolean canHaveCData(XmlElement node)
	{
		XmlElement definition = getDefinitionFor(node.getName());
		if(definition==null)
		{
			return false;
		}
		String cData = definition.getAttributeValueByName("hasCData");
		if(cData==null || cData.equals("false"))
		{
			return false;
		}
		else
		{
			return true;
		}
	}
	
	public boolean canBeRemoved(XmlElement root,XmlElement target)
	{
		if(root==target)
		{
			return false;
		}		
		
		XmlElement parent = getParent(root,target);
		if(parent==null)
		{
			return true;
		}
		XmlElement definition = getDefinitionFor(parent.getName(),target.getName());
		int instances = parent.getElementsByName( target.getName() ).size();
		int	min = getMinimumInstances( definition );
		
		if(instances>min)
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	
	public void remove(XmlElement root,XmlElement target)
	{
		if(canBeRemoved(root,target)==false)
		{
			return;
		}
		if(root!=target)
		{
			XmlElement parent = getParent(root,target);
			if(parent!=null)
			{
				parent.removeChild(target);
			}
		}		
	}
	
	public boolean canBeRemoved(XmlElement root,XmlElement target,String attributeName)
	{
		XmlElement attributeDefinition = schemaDB.findElement("schema/node[@name='"+target.getName()+"']/attribute[@name='"+attributeName+"']");
		
		if(attributeDefinition==null)
		{
			return true;
		}
		String required = attributeDefinition.getAttributeValueByName("required");
		if(required==null || required.equals("false"))
		{
			return true;
		}
		
		return false;
	}
	
	public boolean isParentOrdered(XmlElement root,XmlElement target)
	{
		XmlElement parent = getParent(root,target);
		if(parent==null)
		{
			return false;
		}
		String ordered = getDefinitionFor( parent.getName() ).getAttributeValueByName("ordered");
		if(ordered==null || ordered.equals("true"))
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	
	public String getAttributeTypeFor(String nodeName,String attributeName)
	{
		String type = getAttributeDefinitionFor(nodeName,attributeName).getAttributeValueByName("type");
		if(type==null)
		{
			type="string";
		}
		
		return type;
	}
	
	public String getCDataTypeFor(String nodeName)
	{
		String type = getDefinitionFor(nodeName).getAttributeValueByName("CDataType");
		if(type==null)
		{
			type="string";
		}
		
		return type;
	}	
	
	public XmlElement getDefinitionFor(String nodeName)
	{
		return schemaDB.findElement("schema/node[@name='"+nodeName+"']");		
	}

	private XmlElement getDefinitionFor(String nodeName,String childName)
	{
		return schemaDB.findElement("schema/node[@name='"+nodeName+"']/child[@name='"+childName+"']");		
	}
	
	private XmlElement getAttributeDefinitionFor(String nodeName,String attributeName)
	{
		return schemaDB.findElement("schema/node[@name='"+nodeName+"']/attribute[@name='"+attributeName+"']");		
	}
	
	private int getMinimumInstances(XmlElement definition)
	{
		String minValue = definition.getAttributeValueByName("min");
		if(minValue==null)
		{
			return 0;
		}
		else
		{
			return Integer.parseInt( minValue );
		}
		
	}
	
	private int getMaximumInstances(XmlElement definition)
	{
		String maxValue = definition.getAttributeValueByName("max");
		if(maxValue==null || maxValue.equals("*"))
		{
			return -1;
		}
		else
		{
			return Integer.parseInt( maxValue );
		}
		
	}	
	
	public boolean canMoveUp(XmlElement root,XmlElement target)
	{
		if(isParentOrdered(root,target)==false)
		{
			return false;
		}		
		XmlElement parent = getParent(root,target);
		int index = parent.getElementIndex(target);
		
		if(index>0)
		{
			return true;
		}
		else
		{
			return false;
		}
	}

	public boolean canMoveDown(XmlElement root,XmlElement target)
	{
		if(isParentOrdered(root,target)==false)
		{
			return false;
		}		
		XmlElement parent = getParent(root,target);
		int index = parent.getElementIndex(target);
		
		if(index<parent.getElementCount() && index+1!=parent.getElementCount())
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	
	public void moveDown(XmlElement root,XmlElement target)
	{
		if(isParentOrdered(root,target)==false)
		{
			return;
		}
		XmlElement parent = getParent(root,target);
		int index = parent.getElementIndex(target);
		if(index<parent.getElementCount() && index+1!=parent.getElementCount())
		{
			parent.removeChild(target);
			parent.insertChild(target,index+1);
		}		
	}
	
	public void moveUp(XmlElement root,XmlElement target)
	{
		if(isParentOrdered(root,target)==false)
		{
			return;
		}
		XmlElement parent = getParent(root,target);
		int index = parent.getElementIndex(target);
		if(index>0)
		{
			parent.removeChild(target);
			parent.insertChild(target,index-1);
		}	
	}	
	
	public XmlElement getParent(XmlElement root,XmlElement target)
	{
		if(root==target)
		{
			return target;
		}
		else
		{
			return XmlHelper.getParent(root,target);
		}
	}

	public XmlElement getList(String listName) {
		return schemaDB.findElement("schema/list[@name='"+listName+"']");
	}
}
