/*
 * Created on 17-Oct-2004
 *
 * To change the template for this generated file go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
package org.one.stone.soup.xapp.swing.components;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.InputStream;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.AbstractButton;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JTable;
import javax.swing.event.ChangeEvent;
import javax.swing.plaf.metal.MetalLookAndFeel;
import javax.swing.table.DefaultTableCellRenderer;

import org.one.stone.soup.browser.Browser;
import org.one.stone.soup.xapp.XappComponentStore;
import org.one.stone.soup.xapp.XappPersister;
import org.one.stone.soup.xapp.XappReceiver;
import org.one.stone.soup.xapp.XappRootApplication;
import org.one.stone.soup.xapp.XappSender;
import org.one.stone.soup.xapp.components.XappComponent;
import org.one.stone.soup.xapp.components.spreadsheet.XappSpreadsheet;
import org.one.stone.soup.xapp.components.spreadsheet.XappSpreadsheetModel;
import org.one.stone.soup.xapp.containers.XappContainer;
import org.one.stone.soup.xapp.javascript.JavascriptConsole;
import org.one.stone.soup.xapp.plugin.spreadsheet.XappSpreadsheetPlugin;
import org.one.stone.soup.xml.XmlElement;
import org.one.stone.soup.xml.helper.XmlCloner;
import org.one.stone.soup.xml.stream.XmlLoader;

/*
 * Wet-Wired.com Xui Version Date 17-Oct-2004
 *
 * Copyright 2000-2001 by Wet-Wired.com Ltd.,
 * Portsmouth England
 * This software is OSI Certified Open Source Software
 * This software is covered by an OSI approved open source licence
 * which can be found at http://www.onestonesoup.org/OSSLicense.html
 */
/**
 * @author nikcross
 *
 */
public class XappSwingSpreadsheet extends JTable implements XappContainer,XappSender,XappReceiver,XappPersister,MouseListener,ActionListener, XappSpreadsheet{
	
	private static Color functionCellColor = new Color(220,220,255);
	private XappSwingPopup popup;
	private XmlElement settings;
	
	private class XappSpreadsheetCellRenderer extends DefaultTableCellRenderer 
	{
		private XappSwingSpreadsheet parent;
		XappSpreadsheetCellRenderer(XappSwingSpreadsheet parent)
		{
			this.parent = parent;
		}
		
	    public Component getTableCellRendererComponent
	       (JTable table, Object value, boolean isSelected,
	       boolean hasFocus, int row, int column) 
	    {
	    	XmlElement field = spreadsheet.getField(row,column-1);
	    	Component cell  = null;
	    	
	    	if(field!=null && field.getAttributeValueByName("type")!=null)
	    	{
	    		field = XmlCloner.getClone(field);
	    		field.addAttribute("value",field.getValue());
	    		field.addAttribute("name",parent.getName()+"."+spreadsheet.getColumnName(column)+(row+1));
	    		cell = (Component)XappRootApplication.getComponentFactory().buildInput(parent,field,parent.getName(),new XappComponentStore());
	    		
	    		return cell;
	    	}
	    	else
	    	{
    	        XmlElement xCell = parent.getSpreadsheetModel().getColumnRenderer(column);
	    		
    	        if(xCell!=null)
	    		{
        	        xCell = XmlCloner.getClone(xCell.getElementByIndex(0));
		        	xCell.setAttributeValue("name",parent.getName()+"."+spreadsheet.getColumnName(column)+(row+1));
		    		xCell.addAttribute("value",field.getValue());
		        	cell = (Component)XappRootApplication.getComponentFactory().buildInput(parent,xCell,getName(),XappRootApplication.getComponentStore());
		    		
		        	return cell;	    			
	    		}
	    		else
	    		{
	    			cell = super.getTableCellRendererComponent
	    				(table, value, isSelected, hasFocus, row, column);
	    		}
	    	}
	    	
	        if(isSelected)
	        {
	        	cell.setBackground(Color.lightGray);
	        	cell.setForeground(Color.black);
	        }
	        else if(value.equals(""))
	        {
	        	cell.setBackground( MetalLookAndFeel.getDesktopColor() );
	        	cell.setForeground( MetalLookAndFeel.getSystemTextColor() );
	        }
	        else
	        {
	        	cell.setBackground(Color.white);
	        	cell.setForeground(Color.black);	        
	        }
	        
	        if(hasFocus)
	        {
	        	cell.setBackground(Color.white);
	        	cell.setForeground(Color.black);
	        }
	        else if(isFunctionCell(row,column-1))
	        {
	        	cell.setBackground( functionCellColor );
	        }
	        
	        if(column==0)
	        {
	        	cell = new JButton( value.toString() );
	        	((JButton)cell).setBorderPainted(false);
	        	if(isSelected)
	        	{
	        		cell.setBackground(Color.black);
	        	}
	        }
	        
	        return cell;
	 
	    }
	}
	
	private boolean editing = false;
	private XappSpreadsheetModel spreadsheet;
	private Timer timer;
	
	public XappSwingSpreadsheet(XmlElement data) {
		super();
		setData(data);
		//this.setBackground(new Color(50,50,200));
		this.setForeground(Color.black);
		this.setDefaultRenderer( String.class,new XappSpreadsheetCellRenderer(this) );
		this.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		
		getTableHeader().setReorderingAllowed(false);
		getTableHeader().addMouseListener(this);
		addMouseListener(this);
	}
	
	/* (non-Javadoc)
	 * @see org.one.stone.soup.xapp.components.spreadsheet.XappSpreadsheet#resizeColumn(int)
	 */
	public void resizeColumn(int column)
	{
		String title = (String)this.getColumnModel().getColumn(column).getHeaderValue();
		double width = getGraphics().getFontMetrics().getStringBounds(title,getGraphics()).getWidth();
			
		for(int loop=0;loop<getSpreadsheetModel().getRowCount();loop++)
		{
			String value = getSpreadsheetModel().getValueAt(loop,column).toString();
			double newWidth = getGraphics().getFontMetrics().getStringBounds(value,getGraphics()).getWidth();
			if(newWidth>width)
			{
				width = newWidth;
			}
		}
		
		this.getColumnModel().getColumn(column).setPreferredWidth((int)width+20);
		this.invalidate();
	}
	
	/* (non-Javadoc)
	 * @see org.one.stone.soup.xapp.components.spreadsheet.XappSpreadsheet#getData()
	 */
	public XmlElement getData()
	{
		XmlElement data = spreadsheet.getData();
		XmlElement oldSettings = data.getElementByName("settings");
		if(oldSettings!=null)
		{
			data.removeChild(oldSettings);
		}
		if(settings!=null)
		{
			XmlElement newSettings = XmlCloner.getClone(settings);
			newSettings.setName("settings");
			data.addChild( newSettings );
		}
		return data;
	}
	
	/* (non-Javadoc)
	 * @see org.one.stone.soup.xapp.components.spreadsheet.XappSpreadsheet#getSpreadsheetModel()
	 */
	public XappSpreadsheetModel getSpreadsheetModel()
	{
		return spreadsheet;
	}
	
	/* (non-Javadoc)
	 * @see org.one.stone.soup.xapp.components.spreadsheet.XappSpreadsheet#setData(org.one.stone.soup.xml.XmlElement)
	 */
	public void setData(XmlElement data)
	{
		XmlElement newSettings = data.getElementByName("settings");
		if(newSettings!=null)
		{
			settings = newSettings;
			data.removeChild(newSettings);
			
			applySettings();
		}
		
		spreadsheet = new XappSpreadsheetModel(data,this);
		this.setModel(spreadsheet);
	}
	
	public void add(XappComponent component) {
		//TODO add((Component)component);
	}
	
	public void add(String name, XappComponent component) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.one.stone.soup.xui.XUIContainer#addGroupMember(java.lang.String, javax.swing.AbstractButton)
	 */
	public void addGroupMember(String key, AbstractButton member) {
		// TODO Auto-generated method stub
	}
	/* (non-Javadoc)
	 * @see org.one.stone.soup.xui.XUISender#setReceiver(org.one.stone.soup.xui.XUIReceiver)
	 */
	public void addReceiver(XappReceiver receiver) {
		// TODO Auto-generated method stub
	}
	public void removeReceiver(XappReceiver receiver)
	{
		// TODO
	}	
	/* (non-Javadoc)
	 * @see org.one.stone.soup.xui.XUIReceiver#receiveXUIData(org.one.stone.soup.xml.XmlElement)
	 */
	public void receiveXappData(XmlElement data) {
		
		if(data.getName().equals("popup"))
		{
			if(data.getElementByIndex(0).getValue().equals("showSettings"))
			{
				showSettings();
			}
			else if(data.getElementByIndex(0).getValue().equals("recalculate"))
			{
				recalculate();
			}
		}
		else
		{
			settings = data;
			applySettings();
		}
	}
	
	private void applySettings()
	{
		if(settings==null)
		{
			return;
		}
		boolean autoRecalculate = new Boolean(settings.getElementByName( "autoRecalculate" ).getValue()).booleanValue();
		
		if(timer!=null)
		{
			timer.cancel();
			timer = null;
		}
		
		if(autoRecalculate==true)
		{	
			class Task extends TimerTask
			{
				public void run()
				{
					recalculate();
				}
			}
			
			long recalcPeriod = Long.parseLong(settings.getElementByName( "recalculationPeriod" ).getValue());
			
			timer = new Timer();
			timer.scheduleAtFixedRate(new Task(),0,recalcPeriod);
		}
	}
	
	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		
		JavascriptConsole console = (JavascriptConsole)e.getSource();
		XmlElement function = console.getFunction();
		int row = console.getRow();
		int column = console.getColumn();
		
		editing = false;
		spreadsheet.processFieldFunction(row,column);
		
		recalculate();
	}
	/* (non-Javadoc)
	 * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
	 */
	public void stateChanged(ChangeEvent e) {
		// TODO Auto-generated method stub
	}

	public synchronized void mouseClicked(MouseEvent e) {
		
		XappSpreadsheetPlugin.setCurrentSheetName( this.getName() );
		
		if(e.getSource().equals(getTableHeader()))
		{
			if(e.getButton()==MouseEvent.BUTTON1 && e.getClickCount()>1)
			{
				int column = getTableHeader().getColumnModel().getColumnIndexAtX(e.getX());
				
				if(column==0)
				{
					for(int loop=1;loop<getColumnCount();loop++)
					{
						resizeColumn(loop);
					}
				}
				else
				{
					resizeColumn(column);
				}
			}
			else if(e.getButton()!=MouseEvent.BUTTON1)
			{
				showPopup(e.getX(),e.getY());
			}
			
			return;
		}
		
		int row = getSelectedRow();
		int column = getSelectedColumn()-1;
		
		if(isFunctionCell(row,column) || e.getButton()!=MouseEvent.BUTTON1 )
		{
			editFunctionCell(row,column);
		}
	}

	private void showPopup(int x,int y)
	{
		try{
			if(popup!=null)
			{
				popup.hide();
			}
			
			InputStream iStream = Browser.getInputStream("jar://org/one/stone/soup/xapp/components/spreadsheet/spreadsheet.popup");
			popup = new XappSwingPopup(XmlLoader.load(iStream));
			popup.addReceiver(this);
			popup.show((Component)this,x,y);
			
		}
		catch(Exception e2)
		{
			XappRootApplication.displayException(e2);
		}		
	}
	
	private void showSettings()
	{
		try{
			InputStream iStream = Browser.getInputStream("jar://org/one/stone/soup/xapp/components/spreadsheet/spreadsheet-settings.form");
			XSwingForm form = new XSwingForm(XmlLoader.load(iStream),new XappComponentStore());
			form.addReceiver(this);
			
			JFrame formFrame = new JFrame();
			formFrame.getContentPane().add( form );
			formFrame.pack();
			formFrame.show();
		}
		catch(Exception e2)
		{
			XappRootApplication.displayException(e2);
		}
	}
	
	/* (non-Javadoc)
	 * @see org.one.stone.soup.xapp.components.spreadsheet.XappSpreadsheet#isFunctionCell(int, int)
	 */
	public boolean isFunctionCell(int row,int column)
	{
		if(row<0 || column<0 || row>spreadsheet.getRowCount() || column>spreadsheet.getColumnCount() )
		{
			return false;
		}
		XmlElement field = spreadsheet.getSourceField(row,column);
		if(field==null)
		{
			return false;
		}
		String value = field.getValue();
		
		if(value!=null && value.indexOf(":")!=-1)
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/* (non-Javadoc)
	 * @see org.one.stone.soup.xapp.components.spreadsheet.XappSpreadsheet#editFunctionCell(int, int)
	 */
	public void editFunctionCell(int row,int column)
	{
		editing = true;
		
		if(row<0 || column<0)
		{
			return;
		}
		
		XmlElement function = spreadsheet.getSourceField(row,column);
		new JavascriptConsole(this,function,row,column);
	}
	
	public void mouseEntered(MouseEvent e) {}

	public void mouseExited(MouseEvent e) {}

	public void mousePressed(MouseEvent e)
	{
		
	}

	public void mouseReleased(MouseEvent e)
	{
	}
	
	/* (non-Javadoc)
	 * @see org.one.stone.soup.xapp.components.spreadsheet.XappSpreadsheet#recalculate()
	 */
	public void recalculate()
	{
		if(editing)
		{
			return;
		}
		
		int columns = spreadsheet.getColumnCount();
		int rows = spreadsheet.getRowCount();
		
		for(int loopC=0;loopC<columns;loopC++)
		{
			for(int loopR=0;loopR<rows;loopR++)
			{
				spreadsheet.processFieldFunction(loopR,loopC);
			}
		}
		
		refresh();
	}
	
	/* (non-Javadoc)
	 * @see org.one.stone.soup.xapp.components.spreadsheet.XappSpreadsheet#refresh()
	 */
	public void refresh()
	{
		this.resizeAndRepaint();
	}
	
	public String getValue()
	{
		return null;
	}

	public void addActionListener(ActionListener actionListener) {
		// TODO Auto-generated method stub
		
	}

	public void setTip(String text)
	{
		setToolTipText(text);
	}
}
