package org.one.stone.soup.xapp.xpath;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Vector;

import org.one.stone.soup.browser.Browser;
import org.one.stone.soup.file.FileChangeListener;
import org.one.stone.soup.file.FileWatch;
import org.one.stone.soup.xapp.XappRootApplication;
import org.one.stone.soup.xml.XmlDocument;
import org.one.stone.soup.xml.XmlElement;
import org.one.stone.soup.xml.XmlImportProcessor;
import org.one.stone.soup.xml.XmlParseException;
import org.one.stone.soup.xml.XmlParser;
import org.one.stone.soup.xml.helper.XmlPathFunction;
import org.one.stone.soup.xml.helper.XmlPathSearch;
import org.one.stone.soup.xml.stream.XmlLoader;

public class XappXPathDB implements FileChangeListener{

	private XmlPathSearch delegate;
	private FileWatch fileWatch;
	private Vector listeners = new Vector();
	private Vector subPaths = new Vector();
	
	public class XappSubXPathDB implements XappXPathDBChangeListener
	{
		private String rootPath;
		private XmlPathSearch subDelegate;
		private XappXPathDB db;
		private Vector listeners = new Vector();
		
		private XappSubXPathDB( XappXPathDB db,String rootPath )
		{
			setRootPath(rootPath);
			this.db = db;
			
			subPaths.addElement(this);
			db.addListener(this);
		}

		public String getRootPath()
		{
			return rootPath;
		}
		
		private void setRootPath(String path)
		{
			this.rootPath = path;
			if(rootPath.length()==0 || rootPath==null)
			{
				subDelegate = delegate;			
			}
			else
			{
				subDelegate = new XmlPathSearch( delegate.findElement( rootPath )  );
			}			
		}
		
		private void reFindRoot()
		{
			setRootPath(rootPath);
		}
		
		public void addListener(XappXPathDBChangeListener listener)
		{
			listeners.add(listener);
		}
		
		public void removeListener(XappXPathDBChangeListener listener)
		{
			listeners.remove(listener);
		}
		
		public void notifyListeners()
		{
			for(int loop=0;loop<listeners.size();loop++)
			{
				((XappXPathDBChangeListener)listeners.elementAt(loop)).dbChanged(db);
			}
		}		
		
		public void addProcessor(XmlPathFunction function) {
			subDelegate.addProcessor(function);
		}

		public boolean equals(Object obj) {
			return subDelegate.equals(obj);
		}

		public XmlElement findElement(String query, XmlElement node) {
			return subDelegate.findElement(query, node);
		}

		public XmlElement findElement(String query) {
			return subDelegate.findElement(query);
		}

		public XmlDocument findElements(String query, XmlElement node) {
			return subDelegate.findElements(query, node);
		}

		public XmlDocument findElements(String query) {
			return subDelegate.findElements(query);
		}

		public XmlElement getRoot() {
			return subDelegate.getRoot();
		}

		public void dbChanged(XappXPathDB xPathDB) {
			notifyListeners();
		}
		
	}
	
	public XappXPathDB(String xmlFileName,XappXPathDBChangeListener listener) throws IOException,XmlParseException
	{
		this(xmlFileName);
		
		addListener(listener);
	}
	
	public XappXPathDB(String xmlFileName) throws IOException,XmlParseException
	{
		if(xmlFileName.indexOf(':')>=2)
		{
			loadXmlFromURL(xmlFileName);
			return;
		}
		XmlElement root = loadXml(xmlFileName);
		delegate = new XmlPathSearch( root );
		
		fileWatch = new FileWatch(xmlFileName,1000);
		fileWatch.addListener(this);
	}	
	private void loadXmlFromURL(String xmlFileName) throws XmlParseException,IOException
	{
		XmlParser parser = XmlLoader.getStandardParser();
		parser.addCompiler( new XmlImportProcessor() );

		XmlElement root = parser.parseElement( Browser.getInputStream(xmlFileName) );
		
		delegate = new XmlPathSearch( root );
	}
	
	private XmlElement loadXml(String xmlFileName) throws IOException,XmlParseException
	{
		XmlParser parser = XmlLoader.getStandardParser();
		parser.addCompiler( new XmlImportProcessor() );

		return parser.parseElement( new File(xmlFileName) );
	}
	
	public void addListener(XappXPathDBChangeListener listener)
	{
		listeners.add(listener);
	}
	
	public void removeListener(XappXPathDBChangeListener listener)
	{
		listeners.remove(listener);
	}
	
	public void notifyListeners()
	{
		for(int loop=0;loop<listeners.size();loop++)
		{
			((XappXPathDBChangeListener)listeners.elementAt(loop)).dbChanged(this);
		}
	}
	
	public XappSubXPathDB getSubXPathDB(String rootPath)
	{
		return new XappSubXPathDB(this,rootPath);
	}
	
	public void addProcessor(XmlPathFunction function) {
		delegate.addProcessor(function);
	}

	public boolean equals(Object obj) {
		return delegate.equals(obj);
	}

	public XmlElement findElement(String query, XmlElement node) {
		return delegate.findElement(query, node);
	}

	public XmlElement findElement(String query) {
		return delegate.findElement(query);
	}

	public XmlDocument findElements(String query, XmlElement node) {
		return delegate.findElements(query, node);
	}

	public XmlDocument findElements(String query) {
		return delegate.findElements(query);
	}

	public XmlElement getRoot() {
		return delegate.getRoot();
	}

	public void fileChanged(File file) {
		
		try{
			delegate = new XmlPathSearch( loadXml(file.getAbsolutePath()) );
			for(int loop=0;loop<subPaths.size();loop++)
			{
				((XappSubXPathDB)subPaths.elementAt(loop)).reFindRoot();
			}
			
			notifyListeners();
		}
		catch(Exception e)
		{
			XappRootApplication.displayException(e);
		}
	}
	
}
